{
 THistogramControl v0.1
 Visual component Delphi 3
 April 1998 Andreas Moser
 Email: amoser@amoser.de

 This control alows you to drop a histogram onto your form/panels/etc.

 You just have to assign an existing TImage to the SourceImage property

 Published Properties:
      ApplyToImage:    Apply the result to the sourceimage
      ColorOfLines:    Color of the horz./vert. lines
      ColorOfText:     Color of the displayed text
      ColorPlaneGroup: Assign a RadioGroup to this property then you dont
                       have to change the viewed plane by code
      ColorSpaceGroup: Assign a RadioGroup to this property then you dont
                       have to change the Histogramtype by code

      SourceImage:     The image to be processed
      HistogramType:   The kind of histogram which should be diplayed
                       at designtime/initally
 Properties:
      SourceBitmap:    Instead of a TImage you can use a TBitmap as source

 Methods:
      ClearHistogram:  Clear the canvas of THistogramControl
      MakeHistogram:   Create the histogram by code

 The basic mathematics and drawing functions based on sources
 by Earl F. Glynn, EarlGlynn@att.net, http://infomaster.net/external/efg
}

unit Histogram;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls,JPEG,GFX_BaseDef,
  Gfx_Colors, gfx_colorstatistics;

type
  THistogramControl = class(TCustomControl)
  private
    { Private declarations }
    fOnResize:TNotifyEvent;
    fApplyColorPlane:Boolean;
    fHistType:TColorPlane;
    fLineColor:TColor;
    fImage:TImage;
    fTextColor:TColor;
    fSrcBitmap:TBitmap;
    fSrcImage:TImage;
    fColorSpaceGroup:TRadioGroup;
    fColorPlaneGroup:TRadioGroup;

    procedure SetType(Value:TColorPlane);
    procedure SetLineColor(Value:TColor);
    procedure SetTextColor(Value:TColor);
    procedure SetSourceImage(Value:TImage);
    procedure SetSourceBitmap(Value:TBitmap);
    procedure SetColorSpaceGroup(Value:TRadioGroup);
    procedure SetColorPlaneGroup(Value:TRadioGroup);


    procedure ColorSpaceGroupChanged(Sender:TObject);
    procedure ColorPlaneGroupChanged(Sender:TObject);

    procedure UpdateHistogram(CONST ColorPlane:  TColorPlane);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner:TComponent);override;
    destructor Destroy;Override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure Paint;override;
    procedure WMSize (var Message: TWMSize); message WM_SIZE;
    procedure SetBounds(ALeft : integer;ATop : integer;AWidth : integer;AHeight : integer);override;
    procedure ClearHistogram;
    procedure MakeHistogram;
    property SourceBitmap:TBitmap read fSrcBitmap write SetSourceBitmap;
  published
    property ApplyToImage:Boolean read fApplyColorPlane write fApplyColorPlane default true;
    property Color;
    property ColorOfLines:TColor read fLineColor write SetLineColor default clBlack;
    property ColorOfText:TColor read fTextColor write SetTextColor default clBlack;
    property ColorPlaneGroup:TRadioGroup read fColorPlaneGroup write SetColorPlaneGroup;
    property ColorSpaceGroup:TRadioGroup read fColorSpaceGroup write SetColorSpaceGroup;
    property HistogramType:TColorPlane read fHistType write SetType default cpRGB;

    property SourceImage:TImage read fSrcImage write SetSourceImage;
    { Published declarations }
    property OnClick;
    property OnDblClick;
    property OnResize: TNotifyEvent read FOnResize write FOnResize;
  end;

procedure Register;

implementation

constructor THistogramControl.Create(Aowner:Tcomponent);
begin
     inherited Create(AOwner);
     ControlStyle := ControlStyle +
     [csReplicatable,csopaque,csCaptureMouse,csClickEvents,csDoubleClicks];
     fSrcBitmap:=TBitmap.Create;
     fSrcImage:=nil;
     fHistType:=cpRGB;
     fLineColor:=clBlack;
     fTextColor:=clBlack;
     fImage:=TImage.Create(Self);
     fImage.Parent:=self;
     fImage.Align:=alClient;
     Width:=300;
     Height:=100;
end;

destructor THistogramControl.Destroy;
begin
    fSrcBitmap.Free;
    inherited Destroy;
end;

procedure THistogramControl.Paint;
var
  Rect: TRect;
  FontHeight: Integer;
begin
  inherited Paint;
  // Draw frame and caption only at designtime
  if (csDesigning in ComponentState) and not Assigned(fSrcImage) then
    with inherited Canvas do
    begin
      Pen.Style := psDash;
      Brush.Style := bsSolid;
      Brush.Color:=Color;
      Rectangle(0, 0, Width, Height);
      Brush.Style := bsClear;
      Font := Self.Font;
      FontHeight := TextHeight('W');
      Rect := GetClientRect;
      with Rect do
      begin
        Top := ((Bottom + Top) - FontHeight) div 2;
        Bottom := Top + FontHeight;
      end;
      DrawText(Handle, PChar(Caption), -1, Rect, (DT_EXPANDTABS or
      DT_VCENTER) or DT_CENTER);
    end;
end;

procedure THistogramControl.WMSize (var Message: TWMSize);
begin
  inherited;
  if Assigned(FOnResize) then
    FOnResize (Self);
    UpdateHistogram(fHistType);
end;

procedure THistogramControl.SetBounds(ALeft : integer;ATop : integer;AWidth : integer;AHeight : integer);
begin
  // Dont allow horizontal resize
  AWidth:=300;
  inherited;
end;

procedure THistogramControl.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) then
  begin
    if (Acomponent = fColorSpaceGroup) then fColorSpaceGroup:=nil
    else if (AComponent = fColorPlaneGroup) then fColorPlaneGroup:=nil
    else if (AComponent = fSrcImage) then fSrcImage:=nil;
  end;
end;

procedure THistogramControl.SetType(Value:TColorPlane);
begin
   if fHistType<>value then
   begin
     fHistType:=value;
     UpdateHistogram(fHistType);
   end;
end;

procedure THistogramControl.SetSourceImage(Value:TImage);
begin
   if (not Assigned(Value)) or not (Value.Picture.Graphic is tbitmap) then Exit;
   if fSrcImage<>value then
   begin
      fSrcImage:=value;
      SetBitmapsEql(Value.Picture.Bitmap,fSrcBitmap);
      fSrcBitmap.Canvas.Draw(0,0,Value.Picture.Bitmap);
   end;
end;

procedure THistogramControl.SetSourceBitmap(Value:TBitmap);
begin
   if not Assigned(Value) then Exit;
   SetBitmapsEql(Value,fSrcBitmap);
   fSrcBitmap.Canvas.Draw(0,0,Value);
end;

procedure THistogramControl.SetLineColor(Value:TColor);
begin
   if fLineColor<>value then
   begin
     fLineColor:=value;
     UpdateHistogram(fHistType);
   end;
end;

procedure THistogramControl.SetTextColor(Value:TColor);
begin
   if fTextColor<>value then
   begin
     fTextColor:=value;
     UpdateHistogram(fHistType);
   end;
end;

procedure THistogramControl.SetColorSpaceGroup(Value:TRadioGroup);
begin
  if (fColorSpaceGroup <> value) and (value<>ColorPlaneGroup) then begin
     if (Value=nil) and Assigned(fColorSpaceGroup) then
     begin
        fColorSpaceGroup.OnClick:=nil;
        fColorPlaneGroup:=nil;
     end;
     fColorSpaceGroup:=Value;
     if Assigned(fColorSpaceGroup) then
     begin
       fColorSpaceGroup.Items.Clear;
       with fColorSpacegroup.Items do
       begin
            Add ('RGB');
            Add ('HSV');
            Add ('CMYK');
       end;
       fColorSpaceGroup.OnClick:=ColorSpaceGroupChanged;
     end;
     UpdateHistogram(fHistType);
  end;
end;


procedure THistogramControl.SetColorPlaneGroup(Value:TRadioGroup);
begin
  if (fColorPlaneGroup <> value) and (value<>fColorSpaceGroup) then begin
     if (Value=nil) and Assigned(fColorPlaneGroup) then
     begin
       fColorPlaneGroup.OnClick:=nil;
       fColorSpaceGroup:=nil;
     end;
     fColorPlaneGroup:=Value;
     if Assigned(fColorPlaneGroup) then
     begin
       fColorPlaneGroup.Items.Clear;
       with fColorPlaneGroup.Items do
       begin
         Add ('RGB Composite');
         Add ('Red');
         Add ('Green');
         Add ('Blue');
       end;
       fColorPlaneGroup.OnClick:=ColorPlaneGroupChanged;
       if Assigned(fColorSpaceGroup) then fColorSPaceGroup.Itemindex:=0;
     end;
     UpdateHistogram(fHistType);
  end;
end;

procedure THistogramControl.ColorSpaceGroupChanged(Sender:TObject);
begin
  if Assigned(fColorSpaceGroup)
  and Assigned(fColorPlaneGroup) then
  begin
  fColorPlaneGroup.Items.Clear;
  CASE (Sender as TRadioGroup).ItemIndex OF
    0:  {RGB}
        BEGIN
          WITH fColorPlaneGroup.Items DO
          BEGIN
            Add ('RGB Composite');
            Add ('Red');
            Add ('Green');
            Add ('Blue')
          END
        END;

    1:  {HSV}
        BEGIN
          WITH fColorPlaneGroup.Items DO
          BEGIN
            Add ('Hue');
            Add ('Saturation');
            Add ('Value')
          END
        END;

    2:  {CMY}
        BEGIN
          WITH fColorPlaneGroup.Items DO
          BEGIN
            Add ('Cyan');
            Add ('Magenta');
            Add ('Yellow');
            Add ('Black');
          END
        END
  END;
  fColorPlaneGroup.ItemIndex := 0
  end;
end;

procedure THistogramControl.ColorPlaneGroupChanged(Sender:TObject);
begin
  if Assigned(fColorPlaneGroup) and Assigned(ColorSpaceGroup) then begin
  ClearHistogram;
  fHistType := cpRGB;
  CASE fColorSpaceGroup.ItemIndex OF
    0:  {csRGB}
        BEGIN
          CASE fColorPlaneGroup.ItemIndex OF
            0:  fHistType := cpRGB;
            1:  fHistType := cpRed;
            2:  fHistType := cpGreen;
            3:  fHistType := cpBlue
          END
        END;

    1:  {csHSV}
        BEGIN
          CASE fColorPlaneGroup.ItemIndex OF
            0:  fHistType := cpHueHSV;
            1:  fHistType := cpSaturationHSV;
            2:  fHistType := cpValue
          END
        END;

    2:  {csCMYK}
        BEGIN
          CASE fColorPlaneGroup.ItemIndex OF
            0:  fHistType := cpCyan;
            1:  fHistType := cpMagenta;
            2:  fHistType := cpYellow;
            3:  fHistType := cpBlack;
          END
        END
  END;
  UpdateHistogram(fHistType);
  end;
end;

procedure THistogramControl.UpdateHistogram(CONST ColorPlane:  TColorPlane);
  var
    BitmapProcessed:TBitmap;
    HistogramArray :  THistogram;
BEGIN
  if Assigned(fSrcBitmap) then 
  try
   ClearHistogram;
   if fSrcBitmap.PixelFormat=pf24Bit then
   begin
    GetHistogram(fSrcBitmap, ColorPlane, HistogramArray);
    fImage.Picture.Bitmap.Width:=fImage.Width;
    fImage.Picture.Bitmap.height:=fImage.Height;
    fImage.Picture.Bitmap.Canvas.Brush.Color:=clBtnFace;
    fImage.Picture.Bitmap.Canvas.FillRect(Rect(0,0,fImage.Picture.Bitmap.Width,fImage.Picture.Bitmap.Height));
    DrawHistogram(ColorPlane, HistogramArray, fimage.Picture.Bitmap.Canvas,fLineColor,fTextColor);

    BitmapProcessed := TBitmap.Create;
    try
      BitmapProcessed.Height      := fSrcBitmap.Height;
      BitmapProcessed.Width       := fSrcBitmap.Width;
      BitmapProcessed.PixelFormat := fSrcBitmap.PixelFormat;

      IF   ColorPlane = cpRGB
      THEN GetImagePlane (ColorPlane, TRUE, fSrcBitmap, BitmapProcessed)
      ELSE GetImagePlane (ColorPlane, FALSE, fSrcBitmap, BitmapProcessed);

      //Should the selected colorplane be applied to sourceimage ?
      if fApplyColorPlane then
      begin
        fSrcBitmap.Assign(BitmapProcessed);
        if Assigned(fSrcImage) then fSrcImage.Picture.Bitmap.Assign(BitmapProcessed);
      end;

      FINALLY
      BitmapProcessed.Free
    END
   end;
  FINALLY
  END
END;

procedure THistogramControl.ClearHistogram;
begin
  fImage.Picture.Bitmap.Canvas.Brush.Color:=Color;
  fImage.Picture.Bitmap.Canvas.FillRect(Rect(0, 0,fImage.Picture.Bitmap.Width,fImage.Picture.Bitmap.Height));
  Visible := TRUE;
end;

procedure THistogramControl.MakeHistogram;
begin
 UpdateHistogram(fHistType);
end;
procedure Register;
begin
  RegisterComponents('AM FX Pro', [THistogramControl]);
end;

end.
